;;;-------------------------------------------------------------------------
;;; File FLASH        FC (Flash Copy), FE (Flash erase) command
;;;-------------------------------------------------------------------------	

;;;
;;; The flash chip is AT49F002. Commands are generic, they don't check which
;;; sector will be erased, so care must be taken by the user. For example,
;;; one cannot erase the sector where this command is running (if running
;;; from flash).
;;;
;;; Great care should also be taken when erasing sector 0, since it contains
;;; the reset vector, and may render the board useless (or force a trip to the
;;; eprom burner to recover !)
;;;
;;; Once the monitor has stabilized, a real eprom can be added and run the
;;; monitor code from there (and its reset vector)
;;; 
	
FLASH_MAGIC0	EQU	ROM_LOW+$5555
FLASH_MAGIC1	EQU	ROM_LOW+$2AAA


;;;
;;; FE : FLASH ERASE
;;;
FECMD	LEA	SYNTAX(PC),A0
	BSR	FNEXTF
	BSR	GETA
	MOVE.L	D0,A1			; A1 = address in sector to erase

	CLR.L	D7			; D7 = do not erase
	MOVE.L	A5,A2
FE000	CMP.L	A2,A6			; look for options
	BCS	FE001			; no more...
	CMPI.B	#';',(A2)+
	BNE	FE000
	MOVE.L	A2,A6
	SUBQ.L	#1,A6			; A6 = pointer to ';'
	CMPI.B	#'Y',(A2)+		; looking for Y
	BNE	SYNTAX			; no... error since it's the only option
	MOVEQ	#1,D7			; confirmation.	

FE001	CMPI.L	#ROM_LOW,D0		; check that A1 is in flash space
	BLT	INVALID_DEST
	CMPI.L	#ROM_HIGH,D0
	BGT	INVALID_DEST

	MOVE.L	A1,D0
	BSR	PPHY			; print destination address (A1)

	TST.L	D7			; erase confirmed ?
	BEQ	FE002
	LEA	RAM_FLASH_ERASE_SECTOR,A4
	JSR	(A4)
	;BSR	flash_erase_sector	; do it !
	BRA	MACSBUG

FE002	BSR	FIXBUF			; print 'NOT CONFIRMED'
	LEA	NOTCONFI(PC),A5
	BSR	FIXDADD
	BRA	MSG


	
;;; 
;;; FC: FLASH COPY
;;; 
FCCMD	LEA	SYNTAX(PC),A0
	BSR	FNEXTF
	BSR	GETA
	MOVE.L	D0,A3			; A3 = from begin address

	BSR	FNEXTF
	BSR	GETA
	MOVE.L	D0,A1			; A1 = from end address

	BSR	FNEXTF
	BSR	GETA
	MOVE.L	D0,A2			; A2 = to begin address

	;; verify that destination if flash.
	
	MOVE.L	A2,D0
	CMPI.L	#ROM_LOW,D0
	BLT	INVALID_DEST		; A2 < ROM_LOW

	MOVE.L	A1,D1
	SUB.L	A3,D1			; D1 = size - 1
	BCS	SYNTAX			; END < BEGIN !
	MOVE.L	A2,D0
	ADD.L	D1,D0			; D0 = A2 + size - 1
	ADDQ.L	#1,D0			; D0 = A2 + size
	CMPI.L	#ROM_HIGH,D0
	BGE	INVALID_DEST		; A2 + size > ROM_HIGH

	;; verify that destination is erased

	MOVE.L	A2,A0			; save A2
FC001	MOVE.B	(A0)+,D0		; read one byte from flash
	ADDQ.B	#1,D0			; add 1, so that FF -> 00
	BNE	NOTERA			; not FF !
	SUBQ.L	#1,D1
	BNE	FC001

	;; ok to flash!
	
	MOVE.L	A3,A0
	BSR	P2PHY			; print address (A0) and (A1)
	MOVE.L	A2,D0
	BSR	PPHY			; print destination address (A2)

	MOVE.L	A1,D1
	SUB.L	A3,D1			; D1 = size - 1
	ADDQ.L	#1,D1			; D1 = count

	MOVE.L	A2,A1			; A1 = flash destination
FC002	MOVE.B	(A3),D0			; get one byte in D0
	LEA	RAM_FLASH_PROGRAM_BYTE,A4
	JSR	(A4)
	;BSR	flash_program_byte	; flash it
	MOVE.B	(A1)+,D0		; read from flash
	SUB.B	(A3)+,D0		; subtract from memory
	BNE	FCFAIL			; if not zero, it failed.
	SUBQ.L	#1,D1
	BNE	FC002
	
	BRA	MACSBUG

FCFAIL	BSR	FIXBUF
	LEA	FCFAIM(PC),A5
	BSR	FIXDADD
	MOVE.L	A1,D0
	SUBQ.L	#1,D0
	BSR	PNT6HX
	MOVE.L	#'  WR',(A6)+
	MOVE.L	#'OTE=',(A6)+
	MOVE.B	-(A3),D0
	BSR	PNT2HX
	MOVE.L	#'   R',(A6)+
	MOVE.L	#'EAD=',(A6)+
	MOVE.B	-(A1),D0
	BSR	PNT2HX
	BRA	MSG	

NOTERA
	BSR	FIXBUF
	LEA	NOTERAM(PC),A5
	BSR	FIXDADD
	MOVE.L	A0,D0
	SUBQ.L	#1,D0			; we did (A0)+ when it failed
	BSR	PNT6HX
	BRA	MSG
	
	
INVALID_DEST
	LEA	INVDEST(PC),A5
	BRA	WHAT93

INVDEST	  DC.B	'NOT FLASH MEMORY',EOT
NOTERAM	  DC.B	'NOT ERASED AT ',EOT
FCFAIM	  DC.B	'VERIFICATION ERROR AT ',EOT
NOTCONFI  DC.B	'NOT CONFIRMED',EOT

	DS	0


;;;
;;; copy the following routines to RAM. They cannot be executed from flash (the chip
;;; becomes unresponsive in flashing mode...)
;;; 
init_flash_routines
	LEA	RAM_FLASH_START,A1
	LEA	flash_program_byte(PC),A0
	LEA	flash_end(PC),A2
ifr_00
	MOVE.W	(A0)+,(A1)+
	CMP.L	A2,A0
	BCS.S	ifr_00
	RTS


;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; DO NOT CHANGE THE FOLLOWING CODE !!!
;;;
;;; The assembler (asm68k) on unix, doesn't allow forward references for
;;; DS directives. The size of the following routines are hard-coded in
;;; aram.asm.
;;; 
;;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
	
;;;
;;; Flash one byte in AT49F002
;;;
;;; Uses :
;;;	D0.B : byte to flash
;;; 	A1   : address
;;;
flash_program_byte
	MOVE.B	#$AA,FLASH_MAGIC0
	MOVE.B	#$55,FLASH_MAGIC1
	MOVE.B	#$A0,FLASH_MAGIC0
	MOVE.B	D0,(A1)
	BRA	flash_poll



;;;
;;; Flash erase sector
;;;
;;; Uses :
;;; 	A1	: sector address
;;;
flash_erase_sector
	MOVE.B	#$AA,FLASH_MAGIC0
	MOVE.B	#$55,FLASH_MAGIC1
	MOVE.B	#$80,FLASH_MAGIC0
	MOVE.B	#$AA,FLASH_MAGIC0
	MOVE.B	#$55,FLASH_MAGIC1	
	MOVE.B	#$30,(A1)
	BRA	flash_poll

;;;
;;; Poll the flash to see if the program operation is finished.
;;;
;;; Algorithm : read a value from the device. Successive reads
;;; will toggle bit 6 until the program cycle is finished.
;;;
;;; Uses : D0,D2
;;; 
flash_poll
	MOVE.B	ROM_LOW,D0
	MOVE.B	ROM_LOW,D2
	EOR	D0,D2
	BNE.S	flash_poll
	RTS

flash_end
	
	
;;;
;;; Flash erase chip
;;;
;; flash_erase_chip
;; 	MOVE.B	#$AA,FLASH_MAGIC0
;; 	MOVE.B	#$55,FLASH_MAGIC1
;; 	MOVE.B	#$80,FLASH_MAGIC0
;; 	MOVE.B	#$AA,FLASH_MAGIC0
;; 	MOVE.B	#$55,FLASH_MAGIC1	
;; 	MOVE.B	#$10,FLASH_MAGIC0
;; 	BRA	flash_poll

