;;;
;;; LCD interface
;;;
;;; The LCD is hooked to the 68008 bus, using the synchronous interface.
;;; The asynchronous interface is too fast for LCD interface.
;;;


;;;
;;; Initialize the LCD
;;; - 8 bits mode
;;; - 2 lines
;;; 
lcd_init
	MOVEM.L	D0/A0,-(SP)

	LEA	LCD_INS,A0
	
	MOVE.B	#$38,(A0)		; select 8 bits interface
	MOVEQ	#5,D0
	BSR	delay_x_1ms		; wait > 4.1ms
	MOVE.B	#$38,(A0)
	BSR	delay_1ms	    	; wait > 100us
	MOVE.B	#$38,(A0)
					; now the busy flag can be checked

	MOVEQ	#$06,D0			; entry mode : increment
	BSR	lcd_puti
	MOVEQ	#$0E,D0			; display ON, cursor ON, blink OFF
	BSR	lcd_puti
	MOVEQ	#$01,D0			; clear display
	BSR	lcd_puti
	MOVEQ	#$80,D0			; set CGRAM = 0
	BSR	lcd_puti
	
	MOVEM.L	(SP)+,D0/A0
	RTS

	
;;;
;;; Wait until the LCD is ready to accept data
;;;
;;; If timeout, Z is not set.
lcd_busy
	MOVEM.L	D0-D1,-(SP)
	MOVE.W	#$FFFF,D1		; D1 = max attempts
lcd_bu0
	MOVE.B	LCD_INS,D0		; read status
	ANDI.B	#$80,D0			; busy flag is bit 7
	BEQ.S	lcd_bu1			; 0...not busy
	DBRA	D1,lcd_bu0		; 1...busy, try again
	MOVEQ	#1,D1			; set status.Z = 0
lcd_bu1
	MOVEM.L	(SP)+,D0-D1
	RTS

	
;;;
;;; Send instruction (in D0) to LCD
;;;
lcd_puti
	BSR	lcd_busy
	MOVE.B	D0,LCD_INS
	RTS

;;;
;;; Send char in D0 to the LCD
;;; 
lcd_putc
	BSR	lcd_busy
	MOVE.B	D0,LCD_DATA
	RTS


;;;
;;; Send a string, pointed by A5, null terminated, to LCD
;;;
;;; Uses :
;;; - at return, A5 points to the char after the null.
;;; 
lcd_puts
	MOVE.L	D0,-(SP)
lcd_pu0
	MOVE.B	(A5)+,D0		; get 1 char
	BEQ.S	lcd_pu1			; if zero, we are done
	BSR	lcd_busy		; wait lcd
	MOVE.B	D0,LCD_DATA		; send char
	BRA	lcd_pu0
lcd_pu1
	MOVE.L	(SP)+,D0
	RTS



;;;
;;; LCD line <1-4>
;;;
lcd_line1
	BSR	lcd_busy
	MOVE.B	#$00,LCD_INS
	RTS
	
lcd_line2
	BSR	lcd_busy
	MOVE.B	#$40,LCD_INS
	RTS
	
lcd_line3
	BSR	lcd_busy
	MOVE.B	#$14,LCD_INS
	RTS
	
lcd_line4
	BSR	lcd_busy
	MOVE.B	#$54,LCD_INS
	RTS
	

;;;
;;; Clear the display
;;;
lcd_clear
	BSR	lcd_busy
	MOVE.B	#$01,LCD_INS
	RTS

	
	