;;;
;;; M48T58 support
;;; 

	
;;; Register map
;;;
;;;  |address | d7 | d6 | d5 | d4 | d3 | d2 | d1 | d0 |  BCD format |
;;;  |--------+-------------------+-------------------+-------------|
;;;  | 1FFF   |    10 years       |   year            |  00-99      |
;;;  | 1FFE   | 0  | 0  | 0  | 10M|   month           |  01-12      |
;;;  | 1FFD   | 0  | 0  | 10 date |   date            |  01-31      |
;;;  | 1FFC   | 0  | FT | CB | CEB| 0  |  day         |  01-07      |
;;;  | 1FFB   | 0  | 0  | 10 hour |   hours           |  00-23      |
;;;  | 1FFA   | 0  | 10 minute    |   minutes         |  00-59      |
;;;  | 1FF9   | ST | 10 second    |   seconds         |  00-59      |
;;;  | 1FF8   | W  | R  | S  |  calibration           |  control    |
;;;  |--------+-------------------+-------------------+-------------|
;;;  | 1FF7   | WDS|BMB4|BMB3|BMB2|BMB1|BMB0| RB1| RB0|  watchdog   |
;;;  | 1FF6   | AFE| x  | ABE| x  | x  | x  | x  | x  |  interrupts |
;;;  | 1FF5   |RPT4| x  | 10 date |  alarm date       |  01-31      | alarm date
;;;  | 1FF4   |RPT3| x  | 10 hour |  alarm hours      |  00-23      | alarm hours
;;;  | 1FF3   |RPT2| 10 minutes   |  alarm minutes    |  00-59      | alarm minutes
;;;  | 1FF2   |RPT1| 10 seconds   |  alarm seconds    |  00-59      | alarm seconds
;;;  | 1FF1   | x  | x  | x  | x  | x  | x  | x  | x  |  not used   |
;;;  | 1FF0   |WDF | AF | Z  | BL | Z  | Z  | Z  | Z  |  flags      |
;;;  |--------+-------------------+-------------------+-------------|
;;;
;;; x = don't care    Z = '0' read-only
;;; 
;;; To read the clock :
;;;    - set 'R' to 1, to stop registers update
;;;    - read the registers
;;;    - set 'R' to 0
;;;
;;; To set the clock :
;;;    - set 'W' and 'R' to 1
;;;    - set the registers
;;;    - set 'W' and 'R' to 0
;;;
;;; FT = frequency test, set to 0 for normal operation
;;; ST = STOP bit. Set to 1 to stop internal oscillator
;;;  W = write bit
;;; AF = alarm flag
;;; BL = battery low
;;; WDS = watchdog steering bit
;;; BMB0-BMB4 = watchdog multiplier bits
;;; RB0-RB1 = watchdog resolution bits
;;; AFE = alarm flag enable
;;; ABE = alarm in battery back-up mode enable
;;; RPT1-RPT4 = alarm repeat mode bits
;;;   RPT[4:1] = 1111 : once per second
;;;              1110 : once per minute
;;;              1100 : once per hour
;;;              1000 : once per day
;;;              0000 : once per month
;;; CEB = Century enable bit
;;; CB = Century bit
;;;

T_CONTROL	EQU	M48T58+$1FF8
T_SECONDS	EQU	T_CONTROL+1
T_MINUTES	EQU	T_CONTROL+2
T_HOURS		EQU	T_CONTROL+3
T_DAY		EQU	T_CONTROL+4
T_DATE		EQU	T_CONTROL+5
T_MONTH		EQU	T_CONTROL+6
T_YEAR		EQU	T_CONTROL+7

;;; 
;;; print or set time. Format  HH:MM:SS
;;; 
TICMD	LEA	TIPRT(PC),A0		; where to go if no params
	BSR	FNEXTF
	BSR	get_deci		; get hours
	MOVE.B	D2,TTIME
	CMPI.B	#23,D0
	BHI.S	TIBAD
	CMPI.B	#':',(A5)+
	BNE.S	TIBAD
	BSR	get_deci		; get minutes
	MOVE.B	D2,TTIME+1
	CMPI.B	#59,D0
	BHI.S	TIBAD
	CMPI.B	#':',(A5)+
	BNE.S	TIBAD
	BSR	get_deci		; get seconds
	MOVE.B	D2,TTIME+2
	CMPI.B	#59,D0
	BHI.S	TIBAD	
	BSR	set_time		; entry correct !	
TIPRT	BSR	FIXBUF
	BSR	get_time
	BSR	OUT1CR
	BRA	MACSBUG
TIBAD	LEA	BADTIME(PC),A5
	BRA	WHAT93

;;;
;;; print or get date. Format  MM/DD/YY
;;; 
DACMD	LEA	DAPRT(PC),A0		; where to go if no params
	BSR	FNEXTF
	BSR	get_deci		; get month
	MOVE.B	D2,TTIME
	CMPI.B	#12,D0
	BHI.S	DABAD
	CMPI.B	#'/',(A5)+
	BNE.S	DABAD
	BSR	get_deci		; get date
	MOVE.B	D2,TTIME+1
	CMPI.B	#31,D0
	BHI.S	DABAD
	CMPI.B	#'/',(A5)+
	BNE.S	DABAD
	BSR	get_deci		; get year
	MOVE.B	D2,TTIME+2
	CMPI.B	#99,D0
	BHI.S	DABAD	
	BSR	set_date		; entry correct !
DAPRT	BSR	FIXBUF
	BSR	get_date
	BSR	OUT1CR
	BRA	MACSBUG
DABAD	LEA	BADDATE(PC),A5
	BRA	WHAT93

BADTIME	DC.B	'INVALID TIME',EOT
BADDATE	DC.B	'INVALID DATE',EOT

	DS	0
	
;;; add time in buffer
get_time
	ORI.B	#$40,T_CONTROL		; set 'R' to 1
	LEA	T_HOURS,A0
	MOVE.B	(A0),D0			; hours
	BSR	PNT2HX
	MOVE.B	#':',(A6)+
	MOVE.B	-(A0),D0		; minutes
	BSR	PNT2HX
	MOVE.B	#':',(A6)+
	MOVE.B	-(A0),D0		; seconds
	BSR	PNT2HX
	ANDI.B	#$BF,-(A0)		; set 'R' to 0
	RTS

;;; add date in buffer
get_date
	ORI.B	#$40,T_CONTROL		; set 'R' to 1
	MOVE.B	T_MONTH,D0		; months
	BSR	PNT2HX
	MOVE.B	#'/',(A6)+
	MOVE.B	T_DATE,D0		; date
	BSR	PNT2HX
	MOVE.B	#'/',(A6)+
	MOVE.B	T_YEAR,D0		; years
	BSR	PNT2HX
	ANDI.B	#$BF,T_CONTROL		; set 'R' to 0
	RTS	

;;; set time
set_time
	ORI.B	#$C0,T_CONTROL		; set 'R' and 'W' to 1
	MOVE.B	TTIME,T_HOURS
	MOVE.B	TTIME+1,T_MINUTES
	MOVE.B	TTIME+2,T_SECONDS
	ANDI.B	#$0F,T_DAY		; make sure FT = 0
	ANDI.B	#$3F,T_CONTROL		; set 'R' and 'W' to 0
	RTS

;;; set date
set_date
	ORI.B	#$C0,T_CONTROL		; set 'R' and 'W' to 1
	MOVE.B	TTIME,T_MONTH
	MOVE.B	TTIME+1,T_DATE
	MOVE.B	TTIME+2,T_YEAR
	MOVE.B	#$01,T_DAY
	ANDI.B	#$3F,T_CONTROL		; set 'R' and 'W' to 0
	RTS

;;;
;;; parse decimal number from 2 bytes in (A5)
;;;
;;; - result in D0 (binary) and D2 (BCD)
;;; - D1 destroyed
;;; 
get_deci
	MOVE.B	#10,D1
	BSR.S	get_deci_nibble
	MOVE.B	D0,D2
	ASL.B	#4,D2
	MULU	D0,D1			; D1 = 10 * D0
	BSR.S	get_deci_nibble
	OR.B	D0,D2
	ADD.B	D1,D0			; D0 += D1
	RTS
get_deci_bad
	MOVE.B	#$FF,D0
	RTS


get_deci_nibble
	MOVE.B	(A5)+,D0
	SUBI.B	#'0',D0
	BLT.S	get_deci_bad
	CMPI.B	#9,D0
	BGT.S	get_deci_bad
	RTS
	