;;;
;;; Interface to the PIC UART
;;;


;;;
;;; Initialize uart
;;;
uart_init
	;; XXX nothing for now
	RTS

;;; 
;;; get one char from PIC's UART in D0
;;;
uart_getc
	MOVE.B	PIC_UART_STATUS,D0	; check if something to read
	ANDI.B	#1,D0
	BEQ	uart_getc		; no, loop
	MOVE.B	PIC_UART_DATA,D0	; get the byte
	RTS

;;;
;;; get one char from PIC's UART in D0, with timeout
;;;
;;; return D0 = -1 if timeout
;;;
uart_getc_timeout
	MOVEM.L	D1/D2,-(SP)
	MOVE.W	UART_TIMEOUT,D2
ugt_wait
	MOVE.B	PIC_UART_STATUS,D1
	ANDI.B	#1,D1
	BNE	ugt_ok
	BSR	delay_1ms
	DBRA	D2,ugt_wait
	MOVEQ	#-1,D0			; timeout
	BRA.S	ugt_done
ugt_ok
	CLR.L	D0
	MOVE.B	PIC_UART_DATA,D0
ugt_done
	MOVEM.L	(SP)+,D1/D2
	RTS
	

;;; 
;;; put one char on PIC's UART (char in D0)
;;;
;;; D1 destroyed
;;; 
uart_putc
	MOVE.B	PIC_UART_STATUS,D1	; check if ok to send
	ANDI.B	#2,D1
	BEQ	uart_putc		; not yet, loop
	MOVE.B	D0,PIC_UART_DATA	; send char
	RTS


;;; 
;;; put one char on PIC's UART (char in D0)
;;;
;;; D0 is -1 if timeout
;;; 
uart_putc_timeout
	MOVEM.L	D0/D1/D2,-(SP)
	MOVE.W	UART_TIMEOUT,D2
upt_wait
	MOVE.B	PIC_UART_STATUS,D1
	ANDI.B	#2,D1
	BNE	upt_ok
	BSR	delay_1ms
	DBRA	D2,ugt_wait
	MOVEM.L	(SP)+,D0/D1/D2
	MOVEQ	#-1,D0			; timeout
	RTS
upt_ok
	MOVEM.L	(SP)+,D0/D1/D2
	MOVE.B	D0,PIC_UART_DATA
	RTS
