

;;;
;;; SEND LINE TO UART WITH CR/LF
;;;
OUT1CR	MOVE.B	#CR,(A6)+
	MOVE.B	#LF,(A6)+

;;;
;;; OUTPUT BUFFER TO UART
;;; 
OUTPUT
	MOVEM.L	D0/D1,-(SP)
OUTP2
	CMP.L	A6,A5			; at or beyond end of line ?
	BCS.S	OUTP3			; no, keep going
	MOVEM.L	(SP)+,D0/D1		; else restore registers
	RTS				; done.
OUTP3
	MOVE.B	(A5)+,D0		; grab byte to output
	BSR	uart_putc		; print it
	BRA.S	OUTP2
	

;;;
;;; INPUT LINE FROM UART, no "auto line feed" entry point
;;;
PORTIN1N
	MOVEM.L	D0-D4/D7/A0-A2,-(SP)
	MOVEQ	#1,D7			; no "auto line feed"
	BRA.S	PORTIN1W
	
;;;
;;; INPUT A LINE FROM UART (convert lowercase to uppercase)
;;;
PORTIN1
	MOVEM.L	D0-D4/D7/A0-A2,-(SP)
PORTIN1V
	CLR.L	D7			; "auto line feed"
PORTIN1W
	CLR.L	D3			; flag as port 1

READBUF
	BSR	uart_getc		; get one byte
	TST.B	D0			; check for null
	BEQ.S	READBUF
	BSR	uart_putc		; echo char
	CMPI.B	#LF,D0			; is it LF ?
	BEQ.S	READBUF			; yes.. don't put in buffer

	CMPI.B	#CTLX,D0		; CTRL-X ?
	BNE.S	CHKCTLH			; no.. go check for backspace
	MOVEQ	#CR,D0			; yes 1) send CR
	BSR	uart_putc
	MOVEQ	#LF,D0			;     2) send LF
	BSR	uart_putc
	MOVE.L	A5,A6			;     3) set BUFFER at start again
	BRA	READBUF			;     4) restart this routine
CHKCTLH
	CMPI.B	#CTLH,D0		; CTRL-H ?
	BEQ.S	BS4CTLH			; yes.. go backspace 1 char
	CMPI.B	#DEL,D0			; DEL ?
	BNE.S	CHKCTLD			; no.. go check for CTRL-D
BKSPACE
	CMP.L	A5,A6			; yes.. usually need an extra backspace
	BEQ	READBUF			; if at start buffer, don't go back
	MOVE.B	#CTLH,D0		; backspace cursor
	BSR	uart_putc
	BRA.S	BLANKIT			; bypass next check
BS4CTLH
	DS	0
	CMP.L	A5,A6			; at beginning of buffer ?
	BEQ	READBUF			; yes, don't go back anymore
BLANKIT
	MOVEQ	#$20,D0			; now blank out previous char
	BSR	uart_putc
	MOVE.B	-(A6),D0		; remove last char from buffer
	MOVE.B	#CTLH,D0		; move cursor back
	BSR	uart_putc
	BRA	READBUF			; get next char
CHKCTLD
	CMPI.B	#CTLD,D0		; CTRL-D ?  (reprint)
	BNE.S	CHKCR			; no.. go check for CR
	MOVEQ	#CR,D0			; yes.. print CR
	BSR	uart_putc
	MOVEQ	#LF,D0			; print LF
	BSR	uart_putc
	MOVE.L	A5,-(SP)		; save A5
	BSR	OUTPUT			; print whole buffer
	MOVE.L	(SP)+,A5		; restore
	BRA	READBUF
CHARSAVE
	CMPI.B	#'a',D0			; < 'a' 
	BLT	CHARSAVE2		; yes, don't convert
	CMPI.B	#'z',D0			; > 'z'
	BGT	CHARSAVE2		; yes, don't convert
	ANDI.B	#$DF,D0			; else convert lowercase to uppercase
					;      by clearing bit 5.
CHARSAVE2
	MOVE.B	D0,(A6)+		; save byte into buffer
	MOVE.L	A6,D1			; calculate room used
	SUB.L	A5,D1			; don't use more than BUFFSIZE
	CMPI.W	#BUFFSIZE,D1
	BMI	READBUF			; there is space, get next char
	BRA	BKSPACE			; no space. consider we got CTRL-H
CHKCR
	CMPI.B	#CR,D0			; CR ? (end of command line)
	BNE.S	CHARSAVE		; no, go store this char

	TST.B	D7			; check for "auto line feed"
	BNE.S	NOAUTOLF		; yes.. bypass sending LF
	MOVEQ	#LF,D0			; no.. send LF
	BSR	uart_putc
NOAUTOLF
	DS	0
	MOVEM.L	(SP)+,D0-D4/D7/A0-A2	; restore registers
	RTS
	
	

;;; 
;;;   FIX THE BUFFER A5 & A6 SET TO START OF BUFFER QUE
;;; 
FIXBUF	LEA	BUFFER,A5
	MOVE.L	A5,A6
	RTS
